package org.openxma.addons.ui.tablecombo.mdl.table;
import org.eclipse.nebula.widgets.tablecombo.TableCombo;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Table;

import at.spardat.xma.mdl.table.TableUIDelegateClient;
import at.spardat.xma.mdl.table.TableWMClient;
import at.spardat.xma.page.EventAdapter;

public class NebulaAwareTableUIDelegateClient extends TableUIDelegateClient {

    /**
     * The SWT Nebula table combo (only if a table combo is used: the table_ instance then holds tableCombo_getTable() )
     */
    private TableCombo                      tableCombo_;
    
    public NebulaAwareTableUIDelegateClient(TableWMClient wModel) {
        super(wModel);
    }
    
    @Override
    public Table attachUIInternal(Object control) {
        if (control instanceof TableCombo) {
            Table ta = ((TableCombo)control).getTable();
            tableCombo_ = (TableCombo) control;
            return ta;
        }
        else {
            return super.attachUIInternal(control);
        }
    }

    @Override
    public void addListeners(Object control, EventAdapter adapter) {
        if (control instanceof TableCombo) {
            ((TableCombo) control).addSelectionListener(adapter);
            addTableListeners(adapter,((TableCombo) control).getTable());
        }
        else {
            super.addListeners(control, adapter);
        }
    }
    
    @Override
    protected void handleUIEventInternal(Object event) {
        if (tableCombo_ != null && event instanceof SelectionEvent) {
            SelectionEvent selectionEvent = (SelectionEvent) event;
            // selection event on the table
            if (selectionEvent.widget == tableCombo_) {
                handleTableSelect();}
        }
        else {
            super.handleUIEventInternal(event);
        }
    }
    
    @Override
    public Object getUIControl() {
        if (!isUIAttached()) throw new IllegalStateException ();
        if (tableCombo_ != null)
            return tableCombo_;
        else
            return super.getUIControl();
    }
    
    @Override 
    public void detachUI() {
        super.detachUI();
        if (tableCombo_ != null) {
            tableCombo_ = null;
        }
    }
    
    @Override
    public boolean isEnabled() {
        if (tableCombo_ != null) {
            if (isUIAttached()) {
                enabled_ = tableCombo_.isEnabled();
            }
            return enabled_;
        }
        else {
            return super.isEnabled();
        }
            
    }
    
    @Override
    protected void deselectAllOnWidget () {
        if (tableCombo_ != null)
            tableCombo_.select(-1);
        else
            super.deselectAllOnWidget();
    }
    
    @Override
    protected void setSelectionOnWidget(int[] selIndexes) {
        if (tableCombo_ != null) {
            if (selIndexes.length > 0) {
                deselectAllOnWidget();
            }
            tableCombo_.select(selIndexes[0]);
        }
        else {
            super.setSelectionOnWidget(selIndexes);
        }
        
    }
    
    @Override
    public void setErrorColor (boolean inError) {
    	if (tableCombo_ != null) {
    		wModel_.getPageModelC().getPageEffects().errorStateChanged(getErrorIndicationWidget(),isEditable(),inError);
    	}
    	else {
    		super.setErrorColor(inError);
    	}
    }
    
    @Override
    protected Control getErrorStateWidget() {
    	if (tableCombo_ != null) {
    		return tableCombo_;
    	}
    	else {
    		return super.getErrorStateWidget();
    	}
    }
    
    @Override
    protected Control getErrorIndicationWidget() {
    	if (tableCombo_ != null) {
    		return tableCombo_.getTextControl();
    	}
    	else {
    		return super.getErrorStateWidget();
    	}
    }
}
